% Clear all variables and close all figures to start with a fresh environment
clear all; 

%% Define parameters for the cartpole system
m = 0.1; % Mass of the pendulum
L = 1; % Length of the pendulum
g = 9.81; % Acceleration due to gravity
M = 1; % Mass of the cart
d1 = 1; % Damping coefficient of the cart
d2 = 0.5; % Damping coefficient of the pendulum

%% Linearized system around the equilibrium point (theta = 0)
q_ = (m+M) * g / (M*L); % Pre-calculation for the A matrix
A = [0,1,0,0 ; 0,-d1, -g*m/M,0 ; 0,0,0,1 ; 0,d1/L,q_,-d2]; % State matrix A
B = [0, 1.0/M, 0., -1/(M*L)].'; % Input matrix B
C_angle = [0,0,1,0]; % Output matrix for angle observation
C_cart = [1,0,0,0]; % Output matrix for cart position observation
D = 0; % Direct transmission matrix (not used here)

%% Design of Luenberger observer
p = 2.5*[-10,-2, -5, -20]; % Desired poles for the observer
C = [C_cart; C_angle];  % Combined output matrix for state estimation
K_luen = place(A.',C',p); % Observer gain matrix calculation

%% Define process and observation noise characteristics
q1 = 5e-4; % Process noise variance for cart position
q2 = 1e-3; % Process noise variance for pendulum angle

r1 = 1e-3; % Measurement noise variance for cart position
r2 = 5e-4; % Measurement noise variance for pendulum angle

% Define initial conditions for the system
x0 = [0,0,0.5, .1]; % Initial state vector

% Define initial process noise covariance
P_k_init = .01; % Initial guess for the process noise covariance

%% Design of optimal control (LQR)
% Continuous time LQR design
R_ctrl = 10; % Weight on control effort in the cost function
Q_ctrl = diag([10,5,20,10]); % Weights on states in the cost function

[X,L,K_lqr] = care(A,B,Q_ctrl, R_ctrl); % Solve continuous-time Algebraic Riccati Equation (CARE)
% there exists a more recent iCARE function but be very careful in change
% the code since the order of the outputs is different
Tsim = 1e-3;                     % Simulation time step

% Discrete time LQR design
Ts = 0.01; % Sampling time

Ad = expm(A*Ts); % Discretize state matrix using matrix exponential
Bd = pinv(A)*(Ad-eye(4))*B; % Discretize input matrix

[X,L,K_lqr_d] = dare(Ad,Bd,Q_ctrl, R_ctrl); % Solve discrete-time Algebraic Riccati Equation (DARE)


%% Frequency analysis section (optional, can be used for additional insight)
[b_angle,a_angle] = ss2tf(A,B,C_angle,D); % Transfer function for angle
[b_cart,a_cart] = ss2tf(A,B,C_cart,D); % Transfer function for cart position

sys_angle = tf(b_angle,a_angle); % Create transfer function model for angle
sys_cart = tf(b_cart,a_cart); % Create transfer function model for cart position

% Root locus plots (commented out, but can be used for analysis)
% rlocus(sys_angle)
% rlocus(sys_cart)